@@ -21,13 +21,15 @@ import (
 	"net"
 	"strings"
 
-	"github.com/aws/aws-sdk-go/aws"
-	"github.com/aws/aws-sdk-go/service/ec2"
 	"github.com/golang/glog"
-	machinev1 "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
-	"golang.org/x/net/context"
+
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+
+	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/service/ec2"
+	machinev1 "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
+	"golang.org/x/net/context"
 	"k8s.io/apimachinery/pkg/types"
 	providerconfigv1 "sigs.k8s.io/cluster-api-provider-aws/pkg/apis/awsproviderconfig/v1beta1"
 	awsclient "sigs.k8s.io/cluster-api-provider-aws/pkg/client"
@@ -364,6 +366,20 @@ func extractNodeAddresses(instance *ec2.Instance) ([]corev1.NodeAddress, error)
 			continue
 		}
 
+		// Treating IPv6 addresses as type NodeInternalIP to match what the KNI
+		// patch to the AWS cloud-provider code is doing:
+		//
+		// https://github.com/openshift-kni/origin/commit/7db21c1e26a344e25ae1b825d4f21e7bef5c3650
+		for _, ipv6Address := range networkInterface.Ipv6Addresses {
+			if addr := aws.StringValue(ipv6Address.Ipv6Address); addr != "" {
+				ip := net.ParseIP(addr)
+				if ip == nil {
+					return nil, fmt.Errorf("EC2 instance had invalid IPv6 address: %s (%q)", aws.StringValue(instance.InstanceId), addr)
+				}
+				addresses = append(addresses, corev1.NodeAddress{Type: corev1.NodeInternalIP, Address: ip.String()})
+			}
+		}
+
 		for _, internalIP := range networkInterface.PrivateIpAddresses {
 			if ipAddress := aws.StringValue(internalIP.PrivateIpAddress); ipAddress != "" {
 				ip := net.ParseIP(ipAddress)
